searchNodes=[{"doc":"Defines an auth module. This module provides a simple set of tools for the authorization and authentication required by a typical API through use of JSON web tokens. To begin, we will want to generate a secret from which our tokens will be signed with. There is a convenient mix task available for this: mix authex . gen . secret We should keep this secret as an environment variable. Next, we will want to create our auth module defmodule MyApp.Auth do use Authex def start_link ( opts \\\\ [ ] ) do Authex . start_link ( __MODULE__ , opts , name : __MODULE__ ) end # Callbacks @impl Authex def init ( opts ) do # Add any configuration listed in Authex.start_link/3 secret = System . get_env ( &quot;AUTH_SECRET&quot; ) || &quot;foobar&quot; opts = Keyword . put ( opts , :secret , secret ) { :ok , opts } end @impl Authex def handle_for_token ( % MyApp.User { } = resource , opts ) do { :ok , [ sub : resource . id , scopes : resource . scopes ] , opts } end def handle_for_token ( _resource , _opts ) do { :error , :bad_resource } end @impl Authex def handle_from_token ( token , _opts ) do # You may want to perform a database lookup for your user instead { :ok , % MyApp.User { id : token . sub , scopes : token . scopes } } end end We must then add the auth module to our supervision tree. children = [ MyApp.Auth ] Tokens At the heart of Authex is the Authex.Token struct. This struct is simply a wrapper around the typical JWT claims. The only additional item is the :scopes and :meta key. There are 3 base actions required for these tokens - creating, signing, and verification. Creating We can easily create token structs using the token/3 function. Authex . token ( MyApp.Auth , sub : 1 , scopes : [ &quot;admin/read&quot; ] ) The above would create a token struct for a resource with an id of 1 and with &quot;admin/read&quot; authorization. Signing Once we have a token struct, we can sign it using the sign/3 function to create a compact token binary. This is what we will use for authentication and authorization for our API. token = Authex . token ( MyApp.Auth , sub : 1 , scopes : [ &quot;admin/read&quot; ] ) Authex . sign ( MyApp.Auth , token ) Verifying Once we have a compact token binary, we can verify it and turn it back to an token struct using the verify/3 function. token = Authex . token ( MyApp.Auth , sub : 1 , scopes : [ &quot;admin/read&quot; ] ) compact_token = Authex . sign ( MyApp.Auth , token ) { :ok , token } = Authex . verify ( MyApp.Auth , compact_token ) Callbacks Typically, we want to be able to create tokens from another source of data. This could be something like a User struct. We also will want to take a token and turn it back into a User struct. To do this, we must implement callbacks. For our auth module above, we can convert a user to a token. token = Authex . for_token ( MyApp.Auth , user ) compact_token = Authex . sign ( MyApp.Auth , token ) As well as turn a token back into a user. token = Authex . verify ( MyApp.Auth , compact_token ) user = Authex . from_token ( MyApp.Auth , token ) Repositories Usually, use of JSON web tokens requires some form of persistence to blacklist tokens through their :jti claim. To do this, we must create a repository. A repository is simply a module that adopts the Authex.Repo behaviour. For more information on creating repositories, please see the Authex.Repo documentation. Once we have created our blacklist, we define it in our opts when starting our auth module or in the init/1 callback. During the verification process used by verify/3 , any blacklist defined in our config will be checked against. Please be aware of any performance penatly that may be incurred through use of database-backed repo's without use of caching. Plugs Authex provides a number of plugs to handle the typical authentication and authorization process required by an API using your auth module. For more information on handling authentication, please see the Authex.Plug.Authentication documentation. For more information on handling authorization, please see the Authex.Plug.Authorization documentation.","ref":"Authex.html","title":"Authex","type":"behaviour"},{"doc":"Blacklists a token jti. This uses the blaclist repo defined in the auth config. The key is the :jti claim in the token. Returns :ok on success, or :error on failure. Example Authex . blacklist ( MyAuth , token )","ref":"Authex.html#blacklist/2","title":"Authex.blacklist/2","type":"function"},{"doc":"Checks whether a token jti is blacklisted. This uses the blaclist repo defined in the auth config. The key is the :jti claim in the token. Returns a boolean. Example Authex . blacklisted? ( MyAuth , token )","ref":"Authex.html#blacklisted?/2","title":"Authex.blacklisted?/2","type":"function"},{"doc":"Generates a compact token from a set of claims. This is simply a shortened version of calling token/3 and sign/3 . Options All options are the same available in token/3 and sign/3 .","ref":"Authex.html#compact_token/3","title":"Authex.compact_token/3","type":"function"},{"doc":"Fetches a config value. Example Authex . config ( MyAuth , :secret )","ref":"Authex.html#config/3","title":"Authex.config/3","type":"function"},{"doc":"Gets the current resource from a Plug.Conn . The resource will only be accessible if the conn has been run through the Authex.Plug.Authentication plug. Returns {:ok, resource} or :error .","ref":"Authex.html#current_resource/1","title":"Authex.current_resource/1","type":"function"},{"doc":"Gets the current scope from a Plug.Conn . The scope will only be accessible if the conn has been run through the Authex.Plug.Authorization plug. Returns {:ok, scope} or :error .","ref":"Authex.html#current_scope/1","title":"Authex.current_scope/1","type":"function"},{"doc":"Gets the current scopes from a Plug.Conn . The scopes will only be accessible if the conn has been run through the Authex.Plug.Authentication plug. Returns {:ok, scopes} or :error .","ref":"Authex.html#current_scopes/1","title":"Authex.current_scopes/1","type":"function"},{"doc":"Gets the current token from a Plug.Conn . The token will only be accessible if the conn has been run through the Authex.Plug.Authentication plug. Returns {:ok, token} or :error .","ref":"Authex.html#current_token/1","title":"Authex.current_token/1","type":"function"},{"doc":"Converts a resource into an Authex.Token . This invokes the handle_for_token/2 callback defined in the auth module. Please see the callback docs for further details. Returns {:ok, token} or {:error, reason} Options Please see the options available in c:token/3 . You can also include any additional options your callback might need. Example { :ok , token } = Authex . for_token ( MyAuth , user )","ref":"Authex.html#for_token/3","title":"Authex.for_token/3","type":"function"},{"doc":"Converts an Authex.Token into a resource. This invokes the handle_from_token/2 defined in the auth module. Please see the callback docs for further details. Returns {:ok, resource} or {:error, reason} . Options You can also include any additional options your callback might need. Example { :ok , user } = Authex . from_token ( token )","ref":"Authex.html#from_token/3","title":"Authex.from_token/3","type":"function"},{"doc":"","ref":"Authex.html#c:handle_for_token/2","title":"Authex.handle_for_token/2","type":"callback"},{"doc":"","ref":"Authex.html#c:handle_from_token/2","title":"Authex.handle_from_token/2","type":"callback"},{"doc":"A callback executed when the auth process starts. This should be used to dynamically set any config during runtime - such as the secret key used to sign tokens with. Returns {:ok, opts} or :ignore . Example def init ( opts ) do secret = System . get_env ( &quot;AUTH_SECRET&quot; ) opts = Keyword . put ( opts , :secret , secret ) { :ok , opts } end","ref":"Authex.html#c:init/1","title":"Authex.init/1","type":"callback"},{"doc":"Refreshes an Authex.Token into a new Authex.Token . When using this function, the assumption has already been made that you have verified it with verify/3 . This will extract the following claims from the original token: :sub :iss :aud :scopes :meta It will then take these claims and generate a new token with them. Options Please see the options available at token/3 . Example token = Authex . refresh ( token )","ref":"Authex.html#refresh/3","title":"Authex.refresh/3","type":"function"},{"doc":"Signs a token, creating a compact token. The compact token is a binary that can be used for authentication and authorization purposes. Typically, this would be placed in an HTTP header, such as: Authorization: Bearer mytoken Returns compact_token or raises an Authex.Error . Options :secret - The secret key to sign the token with. :alg - The algorithm to sign the token with - defaults to :hs256 Any option provided would override the default set in the config.","ref":"Authex.html#sign/3","title":"Authex.sign/3","type":"function"},{"doc":"Starts the auth process. Returns {:ok, pid} on success. Returns {:error, {:already_started, pid}} if the auth process is already started or {:error, term} in case anything else goes wrong. Options :secret - The secret used to sign tokens with. :blacklist - A blacklist repo, or false if disabled - defaults to false . :default_alg - The default algorithm used to sign tokens - defaults to :hs256 . :default_iss - The default iss claim used in tokens. :default_aud - The default aud claim used in tokens. :default_ttl - The default time to live for tokens in seconds. :default_jti - The default mfa used to generate the jti claim. Can be false if you do not want to generate one - defaults to {Authex.UUID, :generate, []} .","ref":"Authex.html#start_link/3","title":"Authex.start_link/3","type":"function"},{"doc":"Creates a new token. A token is a struct that wraps the typical JWT claims but also adds a couple new fields. Please see the Authex.Token documentation for more details. Returns an Authex.Token struct. Options :time - The base time (timestamp format) in which to use. :ttl - The time-to-live for the token in seconds or :infinity if no expiration is required. The lifetime is based on the time provided via the options, or the current time if not provided. Example Authex . token ( MyAuth , sub : 1 , scopes : [ &quot;admin/read&quot; ] )","ref":"Authex.html#token/3","title":"Authex.token/3","type":"function"},{"doc":"Unblacklists a token jti. This uses the blaclist repo defined in the auth config. The key is the :jti claim in the token. Returns :ok on success, or :error on failure. Example MyApp.Auth . unblacklist ( token )","ref":"Authex.html#unblacklist/2","title":"Authex.unblacklist/2","type":"function"},{"doc":"Verifies a compact token. Verification is a multi-step process that ensures: The token has not been tampered with. The current time is not before the nbf value. The current time is not after the exp value. The token jti is not included in the blacklist (if provided). If all checks pass, the token is deemed verified. Returns {:ok, token} or {:error, reason} . Options :time - The base time (timestamp format) in which to use. :secret - The secret key to verify the token with. :alg - The algorithm to verify the token with :blacklist - The blacklist module to verify with. Any option provided would override the default set in the config. Example { :ok , token } = Authex . verify ( MyAuth , compact_token )","ref":"Authex.html#verify/3","title":"Authex.verify/3","type":"function"},{"doc":"","ref":"Authex.html#t:alg/0","title":"Authex.alg/0","type":"type"},{"doc":"","ref":"Authex.html#t:option/0","title":"Authex.option/0","type":"type"},{"doc":"","ref":"Authex.html#t:options/0","title":"Authex.options/0","type":"type"},{"doc":"","ref":"Authex.html#t:signer_option/0","title":"Authex.signer_option/0","type":"type"},{"doc":"","ref":"Authex.html#t:signer_options/0","title":"Authex.signer_options/0","type":"type"},{"doc":"","ref":"Authex.html#t:t/0","title":"Authex.t/0","type":"type"},{"doc":"","ref":"Authex.html#t:verifier_option/0","title":"Authex.verifier_option/0","type":"type"},{"doc":"","ref":"Authex.html#t:verifier_options/0","title":"Authex.verifier_options/0","type":"type"},{"doc":"Defines a verification repo. This allows for the creation of persistent storage for blacklists. Blacklists are used to block usage of a token by its :jti key. defmodule MyApp.Auth.Blacklist do @behaviour Authex.Repo @impl Authex.Repo def exists? ( key ) do # Check if the key exists in the repo. end @impl Authex.Repo def insert ( key ) do # Insert the key in the repo. end @impl Authex.Repo def delete ( key ) do # Delete the key from the repo. end end Please be aware of the performance penalty that may be incurred if using blacklists during the auth verification process. This will largely depend on the storage medium used.","ref":"Authex.Repo.html","title":"Authex.Repo","type":"behaviour"},{"doc":"Deletes a binary key from the repo. Returns :ok on success, or :error on error.","ref":"Authex.Repo.html#c:delete/1","title":"Authex.Repo.delete/1","type":"callback"},{"doc":"Checks if a binary key exists in the repo. Returns a boolean, or :error","ref":"Authex.Repo.html#c:exists?/1","title":"Authex.Repo.exists?/1","type":"callback"},{"doc":"Inserts a binary key into the repo. Returns :ok on success, or :error on error.","ref":"Authex.Repo.html#c:insert/1","title":"Authex.Repo.insert/1","type":"callback"},{"doc":"","ref":"Authex.Repo.html#t:key/0","title":"Authex.Repo.key/0","type":"type"},{"doc":"","ref":"Authex.Repo.html#t:t/0","title":"Authex.Repo.t/0","type":"type"},{"doc":"A struct wrapper for token claims. Typically, we shouldnt need to directly interact with this module. Rather, we should use the Authex.token/3 function.","ref":"Authex.Token.html","title":"Authex.Token","type":"module"},{"doc":"Creates a new Authex.Token struct from the given claims and options. Options :time - the base time (timestamp format) in which to use. :ttl - the TTL for the token or :infinity if no expiration is required. Examples Authex.Token . new ( MyApp.Auth , [ sub : 1 ] , [ ttl : 60 ] )","ref":"Authex.Token.html#new/3","title":"Authex.Token.new/3","type":"function"},{"doc":"","ref":"Authex.Token.html#t:claim/0","title":"Authex.Token.claim/0","type":"type"},{"doc":"","ref":"Authex.Token.html#t:claims/0","title":"Authex.Token.claims/0","type":"type"},{"doc":"","ref":"Authex.Token.html#t:compact/0","title":"Authex.Token.compact/0","type":"type"},{"doc":"","ref":"Authex.Token.html#t:option/0","title":"Authex.Token.option/0","type":"type"},{"doc":"","ref":"Authex.Token.html#t:options/0","title":"Authex.Token.options/0","type":"type"},{"doc":"","ref":"Authex.Token.html#t:t/0","title":"Authex.Token.t/0","type":"type"},{"doc":"Generates a secret and prints it to the terminal. mix authex . gen . secret [ length ] By default, mix authex.gen.secret generates a key 64 characters long. The minimum value for length is 32.","ref":"Mix.Tasks.Authex.Gen.Secret.html","title":"Mix.Tasks.Authex.Gen.Secret","type":"task"},{"doc":"A plug to handle authentication. This plug must be passed an auth module in which to authenticate with. Otherwise, it will raise an Authex.Error . With it, we can easily authenticate a Phoenix controller: defmodule MyAppWeb.MyController do use MyAppWeb , :controller plug Authex.Plug.Authentication , with : MyApp.Auth def show ( conn , _params ) do with { :ok , %{ id : id } } &lt;- MyApp.Auth . current_user ( conn ) , { :ok , user } &lt;- MyApp.Users . get ( id ) do render ( conn , &quot;show.json&quot; , user : user ) end end end The plug looks for the Authorization: Bearer mytoken header by default. It will then verify and covert out token into a resource using the provided auth module. We can then access our current resource from the conn using Authex.current_resource/1 . By default, if authentication fails, the plug sends the conn to the Authex.Plug.Unauthorized plug. This plug will put a 401 status into the conn with the body &quot;Unauthorized&quot; . We can configure our own unauthorized plug by passing it as an option to this plug. Options :with - The auth module that will be used for verification and token conversion. :unauthorized - The plug to call when the token is invalid - defaults to Authex.Plug.Unauthorized . :header - The header to extract the token from - defaults to &quot;authorization&quot; .","ref":"Authex.Plug.Authentication.html","title":"Authex.Plug.Authentication","type":"module"},{"doc":"","ref":"Authex.Plug.Authentication.html#t:option/0","title":"Authex.Plug.Authentication.option/0","type":"type"},{"doc":"","ref":"Authex.Plug.Authentication.html#t:options/0","title":"Authex.Plug.Authentication.options/0","type":"type"},{"doc":"A plug to handle authorization. The plug must also only be used after the Authex.Plug.Authentication has been used. With it, we can easily authorize a Phoenix controller: defmodule MyAppWeb.MyController do use MyAppWeb , :controller plug Authex.Plug.Authentication , with : MyApp.Auth plug Authex.Plug.Authorization , permits : [ &quot;user&quot; , &quot;admin&quot; ] def show ( conn , _params ) do with { :ok , %{ id : id } } &lt;- Authex . current_resource ( conn ) , { :ok , user } &lt;- MyApp.Users . get ( id ) do render ( conn , &quot;show.json&quot; , user : user ) end end end The plug checks the scopes of the token and compares them to the :permits passed to the plug. Authorization works by combining the &quot;permits&quot; with the &quot;type&quot; of request that is being made. For example, with our controller above, we are permitting &quot;user&quot; and &quot;admin&quot; access. The show action would be a GET request, and would therefore be a &quot;read&quot; type. Requests are bucketed under the following types: GET - read HEAD - read PUT - write PATCH - write POST - write DELETE - delete So, in order to access the show action, our token would require one of the following scopes: [&quot;user/read&quot;, &quot;admin/read&quot;] . Or, the token would require [&quot;user/write&quot;, &quot;admin/write&quot;] to access the update action. By default, if authorization fails, the plug sends the conn to the Authex.Plug.Forbidden plug. This plug will put a 403 status into the conn with the body &quot;Forbidden&quot; . We can configure our own forbidden plug by passing it as an option to this plug. Options :forbidden - The plug to call when the scopes are invalid - defaults to Authex.Plug.Forbidden . :permits - A list of permits that the token scopes must have at least one of.","ref":"Authex.Plug.Authorization.html","title":"Authex.Plug.Authorization","type":"module"},{"doc":"","ref":"Authex.Plug.Authorization.html#t:option/0","title":"Authex.Plug.Authorization.option/0","type":"type"},{"doc":"","ref":"Authex.Plug.Authorization.html#t:options/0","title":"Authex.Plug.Authorization.options/0","type":"type"},{"doc":"A plug to mark the status as forbidden. This plug is the default used when authorization fails with Authex.Plug.Authorization . It will put a 403 status into the conn with the body &quot;Forbidden&quot; .","ref":"Authex.Plug.Forbidden.html","title":"Authex.Plug.Forbidden","type":"module"},{"doc":"A plug to mark the status as unauthorized. This plug is the default used when authentication fails with Authex.Plug.Authentication . It will put a 401 status into the conn with the body &quot;Unauthorized&quot; .","ref":"Authex.Plug.Unauthorized.html","title":"Authex.Plug.Unauthorized","type":"module"},{"doc":"","ref":"Authex.Error.html","title":"Authex.Error","type":"exception"},{"doc":"Authex is a simple JWT authentication and authorization library for Elixir.","ref":"readme.html","title":"Authex","type":"extras"},{"doc":"The package can be installed by adding authex to your list of dependencies in mix.exs . In addition, we must also add a JSON encoder/decoder. Jason is recommended. But any of these will work: jiffy , jsone , jsx , ojson , Poison . Finally, if you wish to use any of the plug functionality, make sure to add the plug dependency. def deps do [ { :authex , &quot;~&gt; 2.0&quot; } , { :jason , &quot;~&gt; 1.0&quot; } , { :plug , &quot;~&gt; 1.0&quot; } ] end","ref":"readme.html#installation","title":"Authex - Installation","type":"extras"},{"doc":"See HexDocs for additional documentation.","ref":"readme.html#documentation","title":"Authex - Documentation","type":"extras"},{"doc":"To get started, we must define our auth module: defmodule MyApp.Auth do use Authex def start_link ( opts \\\\ \\\\ [ ] ) do Authex . start_link ( __MODULE__ , opts , name : __MODULE__ ) end # Callbacks @impl Authex def init ( opts ) do # Add any configuration listed in Authex.start_link/3 secret = System . get_env ( &quot;AUTH_SECRET&quot; ) || &quot;foobar&quot; opts = Keyword . put ( opts , :secret , secret ) { :ok , opts } end @impl Authex def handle_for_token ( % MyApp.User { } = resource , opts ) do { :ok , [ sub : resource . id , scopes : resource . scopes ] , opts } end def handle_for_token ( _resource , _opts ) do { :error , :bad_resource } end @impl Authex def handle_from_token ( token , _opts ) do # You may want to perform a database lookup for your user instead { :ok , % MyApp.User { id : token . sub , scopes : token . scopes } } end end And add it to your supervision tree: children = [ MyApp.Auth ] We can then create, sign, and verify tokens: token = Authex . token ( MyApp.Auth , sub : 1 , scopes : [ &quot;admin/read&quot; ] ) compact_token = Authex . sign ( MyApp.Auth , token ) { :ok , token } = Authex . verify ( MyApp.Auth , compact_token ) We can also convert resources to and from tokens. token = Authex . for_token ( MyApp.Auth , user ) compact_token = Authex . sign ( MyApp.Auth , token ) { :ok , token } = Authex . verify ( MyApp.Auth , compact_token ) { :ok , user } = Authex . from_token ( MyApp.Auth , token ) Please check out the documentation for more advanced features.","ref":"readme.html#example","title":"Authex - Example","type":"extras"},{"doc":"Easy to integrate with almost any app. Handles both authentication + authorization. Convert data to and from tokens. Handle persistence for things like blacklists. Batteries included for plug integration.","ref":"readme.html#features","title":"Authex - Features","type":"extras"}]